'use strict';

var _const = require('./data/const.js');
var Options = require('./Options.js');

/*
 * -------------------------------------------------------------------------------
 *  Generate feedback ---------------------------------------------------------------
 * -------------------------------------------------------------------------------
 */

class Feedback {
  constructor() {
    this.defaultFeedback = {
      warning: '',
      suggestions: []
    };
    this.setDefaultSuggestions();
  }

  setDefaultSuggestions() {
    this.defaultFeedback.suggestions.push(Options.translations.suggestions.useWords, Options.translations.suggestions.noNeed);
  }

  getFeedback(score, sequence) {
    if (sequence.length === 0) {
      return this.defaultFeedback;
    }

    if (score > 2) {
      return {
        warning: '',
        suggestions: []
      };
    }

    const extraFeedback = Options.translations.suggestions.anotherWord;
    const longestMatch = this.getLongestMatch(sequence);
    let feedback = this.getMatchFeedback(longestMatch, sequence.length === 1);

    if (feedback !== null && feedback !== undefined) {
      feedback.suggestions.unshift(extraFeedback);

      if (feedback.warning == null) {
        feedback.warning = '';
      }
    } else {
      feedback = {
        warning: '',
        suggestions: [extraFeedback]
      };
    }

    return feedback;
  }

  getLongestMatch(sequence) {
    let longestMatch = sequence[0];
    const slicedSequence = sequence.slice(1);
    slicedSequence.forEach(match => {
      if (match.token.length > longestMatch.token.length) {
        longestMatch = match;
      }
    });
    return longestMatch;
  }

  getMatchFeedback(match, isSoleMatch) {
    switch (match.pattern) {
      case 'dictionary':
        return this.getDictionaryMatchFeedback(match, isSoleMatch);

      case 'spatial':
        return this.getSpatialMatchFeedback(match);

      case 'repeat':
        return this.getRepeatMatchFeedback(match);

      case 'sequence':
        return {
          warning: Options.translations.warnings.sequences,
          suggestions: [Options.translations.suggestions.sequences]
        };

      case 'regex':
        if (match.regexName === 'recentYear') {
          return {
            warning: Options.translations.warnings.recentYears,
            suggestions: [Options.translations.suggestions.recentYears, Options.translations.suggestions.associatedYears]
          };
        }

        break;

      case 'date':
        return {
          warning: Options.translations.warnings.dates,
          suggestions: [Options.translations.suggestions.dates]
        };

      default:
        return {
          warning: '',
          suggestions: []
        };
    }

    return {
      warning: '',
      suggestions: []
    };
  }

  getSpatialMatchFeedback(match) {
    let warning = Options.translations.warnings.keyPattern;

    if (match.turns === 1) {
      warning = Options.translations.warnings.straightRow;
    }

    return {
      warning,
      suggestions: [Options.translations.suggestions.longerKeyboardPattern]
    };
  }

  getRepeatMatchFeedback(match) {
    let warning = Options.translations.warnings.extendedRepeat;

    if (match.baseToken.length === 1) {
      warning = Options.translations.warnings.simpleRepeat;
    }

    return {
      warning,
      suggestions: [Options.translations.suggestions.repeated]
    };
  }

  getDictionaryMatchFeedback(match, isSoleMatch) {
    const warning = this.getDictionaryWarning(match, isSoleMatch);
    const suggestions = [];
    const word = match.token;

    if (word.match(_const.START_UPPER)) {
      suggestions.push(Options.translations.suggestions.capitalization);
    } else if (word.match(_const.ALL_UPPER_INVERTED) && word.toLowerCase() !== word) {
      suggestions.push(Options.translations.suggestions.allUppercase);
    }

    if (match.reversed && match.token.length >= 4) {
      suggestions.push(Options.translations.suggestions.reverseWords);
    }

    if (match.l33t) {
      suggestions.push(Options.translations.suggestions.l33t);
    }

    return {
      warning,
      suggestions
    };
  }

  getDictionaryWarning(match, isSoleMatch) {
    let warning = '';
    const dictName = match.dictionaryName;

    if (dictName === 'passwords') {
      warning = this.getDictionaryWarningPassword(match, isSoleMatch);
    } else if (dictName.includes('wikipedia')) {
      warning = this.getDictionaryWarningWikipedia(match, isSoleMatch);
    } else if (dictName === 'lastnames' || dictName.toLowerCase().includes('firstnames')) {
      warning = this.getDictionaryWarningNames(match, isSoleMatch);
    }

    return warning;
  }

  getDictionaryWarningPassword(match, isSoleMatch) {
    let warning = '';

    if (isSoleMatch && !match.l33t && !match.reversed) {
      if (match.rank <= 10) {
        warning = Options.translations.warnings.topTen;
      } else if (match.rank <= 100) {
        warning = Options.translations.warnings.topHundred;
      } else {
        warning = Options.translations.warnings.common;
      }
    } else if (match.guessesLog10 <= 4) {
      warning = Options.translations.warnings.similarToCommon;
    }

    return warning;
  }

  getDictionaryWarningWikipedia(match, isSoleMatch) {
    let warning = '';

    if (isSoleMatch) {
      warning = Options.translations.warnings.wordByItself;
    }

    return warning;
  }

  getDictionaryWarningNames(match, isSoleMatch) {
    if (isSoleMatch) {
      return Options.translations.warnings.namesByThemselves;
    }

    return Options.translations.warnings.commonNames;
  }

}

module.exports = Feedback;
//# sourceMappingURL=Feedback.js.map
